#include <linux/fs.h>
#include <linux/device.h>
#include <linux/version.h>
#include <linux/miscdevice.h>
#include "lbb_kernel.h"
#include "lbb_cdev_debug.h"
#define MSG_BUF_SIZE (4096)
#define DEVICE_NAME "lbb_cdev"
#define CLASS_NAME "lbb"
static int lbb_char_major=-1;
atomic_t lbb_char_open_count;
static struct class * lbb_char_class=NULL;
static char msg_buf[MSG_BUF_SIZE]={0};
static unsigned short msg_head=0;
static unsigned short msg_tail=0;
static unsigned short msg_overflow=0;
static int lbb_cdev_open(struct inode *,struct file *);
static int lbb_cdev_release(struct inode *,struct file *);
static ssize_t lbb_cdev_read(struct file *,char *,size_t,loff_t *);
static ssize_t lbb_cdev_write(struct file *,const char *,size_t,loff_t *);
static struct file_operations fops=
{
.read=lbb_cdev_read,.write=lbb_cdev_write,.open=lbb_cdev_open,.release=lbb_cdev_release,};
static struct miscdevice lbb_misc_device=
{
.minor=0,.name=DEVICE_NAME,.fops=&fops,.mode=0666,};
int lbb_cdev_init(void)
{
lbb_char_major=register_chrdev(0,DEVICE_NAME,&fops);
if (lbb_char_major < 0)
{
PrintLog(0,KERN_ALERT "LBB cdev failed to register a major number\n");
return lbb_char_major;
}
PrintLog(0,KERN_INFO "LBB cdev registered. major %d\n",lbb_char_major);
lbb_char_class=class_create(THIS_MODULE,CLASS_NAME);
if (IS_ERR(lbb_char_class))
{
unregister_chrdev(lbb_char_major,DEVICE_NAME);
PrintLog(0,KERN_ALERT "LBB cdev failed to register device class\n");
return PTR_ERR(lbb_char_class);
}
PrintLog(0,KERN_INFO "LBB cdev device class registered correctly\n");
misc_register(&lbb_misc_device);
atomic_set(&lbb_char_open_count,0);
PrintLog(0,KERN_INFO "LBB cdev device created\n");
return 0;
}
void lbb_cdev_exit(void)
{
device_destroy(lbb_char_class,MKDEV(lbb_char_major,0));
class_unregister(lbb_char_class);
misc_deregister(&lbb_misc_device);
PrintLog(0,KERN_INFO "LBB cdev device destroy\n");
}
static int lbb_cdev_open(struct inode * inodep,struct file * filep)
{
atomic_inc(&lbb_char_open_count);
return 0;
}
static ssize_t lbb_cdev_read(struct file *filep,char *buffer,size_t len,loff_t *offset)
{
int rcod;
size_t size=0,total=0;
unsigned short tmp,fix_head;
if (!len) return total;
if (msg_overflow)
{
strcpy(buffer,"*** Overflow message buffer\n");
msg_tail=strlen(buffer);
msg_head=0;
msg_overflow=0;
}
fix_head=msg_head;
if (msg_tail > fix_head)
{
size=MSG_BUF_SIZE-msg_tail;
if (size > len) size=len;
rcod=x_copy_to_user(buffer+total,msg_buf+msg_tail,size);
if (rcod) goto err_exit;
len-=size;
total+=size;
tmp=msg_tail+size;
if (tmp >=MSG_BUF_SIZE) tmp-=MSG_BUF_SIZE;
msg_tail=tmp;
}
if (!len) return size;
if (msg_tail !=fix_head)
{
size=fix_head-msg_tail;
if (size > len) size=len;
rcod=x_copy_to_user(buffer+total,msg_buf+msg_tail,size);
if (rcod) goto err_exit;
len-=size;
total+=size;
tmp=msg_tail+size;
if (tmp >=MSG_BUF_SIZE) tmp-=MSG_BUF_SIZE;
msg_tail=tmp;
}
return total;
err_exit:
PrintLog(0,KERN_INFO "LBB cdev: Failed copy_to_user\n");
return-EFAULT;
}
static ssize_t lbb_cdev_write(struct file *filep,const char *buffer,size_t len,loff_t *offset)
{
return 0;
}
static int lbb_cdev_release(struct inode * inodep,struct file *filep)
{
atomic_dec(&lbb_char_open_count);
return 0;
}
int lbb_cdev_set(char * buffer,size_t len)
{
ssize_t i;
unsigned short tmp;
if (msg_overflow) return 0;
for (i=0;i < len;++i)
{
msg_buf[msg_head]=buffer[i];
tmp=msg_head+1;
if (tmp==MSG_BUF_SIZE) tmp=0;
if (tmp==msg_head)
{
msg_overflow=1;
break;
}
msg_head=tmp;
}
return msg_overflow;
}
