#ifndef LBB_KERNEL_H
#define LBB_KERNEL_H
#include "common.h"
struct kobject;
struct inode;
struct file;
struct bio;
struct bio_set;
struct lbb_dev_info;
struct kobj;
struct block_device;
unsigned long get_xsecond(void);
struct bio_set * x_bioset_create(void);
void x_bioset_free(struct bio_set * bset);
void x_bio_reset(struct bio * bio);
void x_bio_copy_dev(struct bio * dst,struct bio * src);
void x_bio_set_dev(struct bio * dst,struct block_device * bdev);
struct gendisk * x_bio_get_disk(struct bio * dst);
void x_bio_set_sector(struct bio * bio,sector_t sector);
sector_t x_bio_get_sector(const struct bio * bio);
void x_bio_abort(struct bio * bio,int errnum);
struct bio * x_bio_split(struct bio * bio,int sectors,struct bio_set * bs);
void x_bio_free_all(struct bio * bio);
unsigned long x_copy_to_user(void * dst,const void * src,unsigned long size);
unsigned long xround_up(unsigned long val,unsigned int div);
unsigned long xround_down(unsigned long val,unsigned int div);
int lbb_store_create_file(const char * name,size_t file_size,struct inode ** inode,struct file ** filp);
int lbb_store_remove_file(struct file * filp);
size_t lbb_store_bmap(struct inode * inode,size_t pos);
struct block_device * lbb_store_get_file_bdev(struct inode * inode);
typedef struct fake_spin_lock * spin_lock_ptr;
spin_lock_ptr x_spin_lock_create(const char * name);
void x_spin_lock_destroy(spin_lock_ptr lock);
void x_spin_lock(spin_lock_ptr lock,unsigned int info);
void x_spin_unlock(spin_lock_ptr lock);
void x_spin_lock_irq(spin_lock_ptr lock,unsigned int info);
void x_spin_unlock_irq(spin_lock_ptr lock);
int x_spin_trylock(spin_lock_ptr lock);
typedef struct fake_rw_lock * sprw_lock_ptr;
sprw_lock_ptr x_rw_lock_create(const char * name);
void x_rw_lock_destroy(sprw_lock_ptr rwlk);
void x_rw_read_lock(sprw_lock_ptr rwlk);
void x_rw_read_unlock(sprw_lock_ptr rwlk);
void x_rw_write_lock(sprw_lock_ptr rwlk);
void x_rw_write_unlock(sprw_lock_ptr rwlk);
void * xkmem_cache_create(const char * name,unsigned int size);
void xkmem_cache_destroy(void * cache);
void * xkmem_cache_zalloc(void * cache);
void xkmem_cache_free(void * cache,void * p);
int xsprintf(char * str,const char * format,... );
size_t xstrlen( const char * str);
void * xmemset( void * ptr,int value,size_t len );
void * xmemcpy(void *dst,const void *src,size_t len );
char * xstrcpy (char * dst,const char * src);
char * xstrncpy (char *dst,const char *src,size_t len);
void * xkmalloc(size_t size);
void xkfree( const void *);
int xstrncmp(const char * str1,const char * str2,size_t len);
void xatomic_add(int i,atomic_t * v);
void xatomic_sub(int i,atomic_t * v);
void xatomic_inc(atomic_t * v);
void xatomic_dec(atomic_t * v);
int xatomic_read(atomic_t * v);
void xatomic_set(atomic_t * v,int i);
int xatomic_dec_and_test(atomic_t * v);
struct semaphore * x_sema_create(int value);
void x_sema_destroy(struct semaphore * sem);
void x_down(struct semaphore * sem);
void x_up(struct semaphore * sem);
typedef struct timer_list x_timer;
x_timer * x_create_timer(void (* callback)(unsigned long),unsigned long data);
void x_destroy_timer(x_timer * timer);
void x_set_timer(x_timer * timer,unsigned long expires);
unsigned long xmsleep_interruptible(unsigned int msecs);
struct block_device * lbb_blockdev_open(char * dev_path,unsigned long mode,void * holder);
unsigned long xbdev_orig_mode(void);
unsigned long xbdev_save_mode(void);
void xblkdev_put(struct block_device * bdev,unsigned int mode);
int x_register_blkdev(unsigned int major,const char * name);
void x_unregister_blkdev(unsigned int major,const char * name);
int check_is_directory(const char * path);
int check_is_blockdev(const char * dev_path);
#endif
